/**
 * @file main.c
 * @author Thomas Weinhold
 * @date 14.02.2011
 * @version 0.1
 *
 * A brief example for the uAC, designed for an ATMega644P.
 * To port this example, just edit this file.
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <uAC.h>
#include <avr/pgmspace.h>

//!UASRT Rx interrupt.
ISR(USART0_RX_vect)
{
	//!Hand over the char to uAC
	uac_rx(UDR0);
}

//!A console test command.
void TestCMD (int argc, char *argv[])
{
	uac_printf("Hello World!\n");
	int i;
	for (i=0;i<argc;i++)
	{
		uac_printf("%i: %s\n",i,argv[i]);
	}

}

int main (void)
{
	//!Init USART
	//!115k2 8N1
	UCSR0A |= (1<<U2X0);
	UCSR0B |= (1<<RXCIE0) | (1<<RXEN0) | (1<<TXEN0);
	UCSR0C |= (1<<UCSZ01) | (1<<UCSZ00);
	UBRR0 = 16;

	//!Init the uAC
	uac_init();
	//!Attach the test command
	uac_attach("test",TestCMD);

	//!don't forget enabling interrupts
	sei();

	//!print something
	uac_printf("Floats: %f %3f\n",123.456789,-76.54321);
	static char TestString[] PROGMEM = "This is a very long text.\n\
It can demonstrate how many ram is saved,\
if such strings are stored in the flash only.\n\
This can be done by including pgmspace.h and use the PROGMEM attribute.\n\
Don't forget to declare the string as static, else it won't work!.\n\
You can try to delete the PROGMEM attribute at this string.\n\
After compiling you will see the Data value will be much higher.\n\
Also interesting is, that both printf-functions are trying to \
print this string,\nbut only the right one is able to do it.\n\
Think about, that long text need much output buffer!!!\n";

	uac_printf_P(TestString);
	uac_printf(TestString);


	//!The main-loop.
	for(;;)
	{
		//!The uAC_Task() must be called periodically
		uac_task();
		//!If there are outgoing chars, send them
		if (uac_txavailable() && (UCSR0A & (1<<UDRE0))) UDR0=uac_tx();
	}
}
